/* Copyright 2009 Jetro Lauha - http://iki.fi/jetro/
 *
 * $Id: MainState.cpp 6 2009-04-20 00:54:02Z tonic $
 * $Revision: 6 $
 */

#include <assert.h>
#include <algorithm>

#include "main.h"
#include "MainState.h"

using namespace irr::core;


#define GAME_SPEED  0.1f


#define LEVEL_COUNT 2

enum SAMPLE_ID
{
    SAMPLE_PICKUP = 0,
    SAMPLE_END,
    SAMPLE_DEATH,
    SAMPLE_COUNT
};

// samples generated with the great sfxr!
static char *sSampleFileNames[SAMPLE_COUNT] =
{
    "data/pickup.ogg",
    "data/end.ogg",
    "data/death.ogg"
};

#define AUDIO_CHANNELS 32

static MainState *sMainState;

static FMUSIC_MODULE *sMusics[LEVEL_COUNT] = { 0 };
static FMUSIC_MODULE *sMusic = 0;
static FMUSIC_MODULE *sTitleMusic = 0;
static FSOUND_SAMPLE *sSamples[SAMPLE_COUNT];



MainState::MainState()
{
}


MainState::~MainState()
{
}


static bool sStringStartsWith(const c8 *str, const c8 *beginning)
{
    irr::core::stringc s(str), b(beginning);
    return s.subString(0, b.size()) == b;
}


/* // just abandoned the callback things since they come in advance (due to buffering?)
void F_CALLBACKAPI sMusicOrderCallback(FMUSIC_MODULE *mod, unsigned char param)
{
    // remember: callback comes from FMOD mixer thread and should be lightweight
    if (sMainState)
        sMainState->newMusicOrder((int)param);
}

void F_CALLBACKAPI sMusicRowCallback(FMUSIC_MODULE *mod, unsigned char param)
{
    // remember: callback comes from FMOD mixer thread and should be lightweight
    if (sMainState)
        sMainState->newMusicRow((int)param);
}

void MainState::newMusicOrder(int order)
{
    // remember: callback comes from FMOD mixer thread and should be lightweight
}

void MainState::newMusicRow(int row)
{
    // remember: callback comes from FMOD mixer thread and should be lightweight
    ++mPendingRowUpdates;
    mSongRow = row;
}
*/


bool MainState::init()
{
    Graphics *g = App::getInstance()->getGraphics();

    sMainState = this;
    mTitleUIActive = true;
    mInGame = false;

    if (!FSOUND_Init(44100, AUDIO_CHANNELS, 0))
    {
        gErrorMessage("Audio error", "Unable to initialize sound! (%d)",
                      FSOUND_GetError());
    }
    for (u32 a = 0; a < SAMPLE_COUNT; ++a)
    {
        sSamples[a] = FSOUND_Sample_Load(FSOUND_FREE, sSampleFileNames[a], FSOUND_LOOP_OFF, 0, 0);
        if (!sSamples[a])
            gErrorMessage("Audio error", "Can't load %s", sSampleFileNames[a]);
    }
    for (int a = 0; a < LEVEL_COUNT; ++a)
    {
        char buf[100];
        sprintf(buf, "data/music%d.it", a);
        sMusics[a] = FMUSIC_LoadSong(buf);
        if (!sMusics[a])
            gErrorMessage("Audio error", "Can't load music");
    }
    sTitleMusic = FMUSIC_LoadSong("data/title.oxm");
    if (!sTitleMusic)
        gErrorMessage("Audio Error", "Can't load title music");
    sMusic = 0;
    //FMUSIC_SetOrderCallback(sMusic, sMusicOrderCallback, 1);
    //FMUSIC_SetRowCallback(sMusic, sMusicRowCallback, 1);
    FMUSIC_PlaySong(sTitleMusic);

    mPendingRowUpdates = 0;
    mSongPrevRow = -1;
    mSongRow = 0;
    mDied = false;

    mKeyLeft = mKeyRight = false;

    mFont = new BMFont(g, "data/droid30.fnt");
    if (!mFont->init())
        return false;

    // Make sure these are in synch with IMAGE_ID
    mImages.push_back(g->loadImage("data/button.png"));
    mImages.push_back(g->loadImage("data/button-hot.png"));
    mImages.push_back(g->loadImage("data/button-active.png"));
    mImages.push_back(g->loadImage("data/button-hot-active.png"));

    mButtonStyle = new IMGUIImageButtonStyle(mFont, mImages[IMG_BUTTON], mImages[IMG_BUTTON_HOT], mImages[IMG_BUTTON_ACTIVE], mImages[IMG_BUTTON_HOT_ACTIVE]);

    mLabelText = "";

    mDevice = App::getInstance()->getIrrDevice();
    mVideo = App::getInstance()->getIrrVideoDriver();
    mSmgr = App::getInstance()->getIrrSceneManager();


    //mCamera = mSmgr->addCameraSceneNodeMaya(0, -50, 10, 30);
    mCamera = mSmgr->addCameraSceneNode();
    mCamera->setNearValue(0.1f);
    mCamera->setFarValue(1200);
    mCamera->setPosition(core::vector3df(-10, 20, 0));
    mCamera->setTarget(core::vector3df(-25, 10, 0));
    mCamera->setUpVector(core::vector3df(0, 1, 0));
    mSmgr->setActiveCamera(mCamera);

    video::ITexture *skyDomeTex = mVideo->getTexture("data/sky.png");
    /*
    scene::ISceneNode *skyDomeNode = mSmgr->addSkyDomeSceneNode(skyDomeTex, 16, 8, 1, 2);
    //skyDomeNode->getMaterial(0).setFlag(video::EMF_BILINEAR_FILTER, false);
    skyDomeNode->getMaterial(0).getTextureMatrix(0).setScale(core::vector3df(4, 2, 1));
    //skyDomeNode->getMaterial(0).setFlag(video::EMF_FOG_ENABLE, true);
    */

    //mSmgr->addCubeSceneNode(10, 0, 0, vector3df(-100, 0, 0));  // ref pos test cube
    mLightSN[0] = mSmgr->addLightSceneNode(0, core::vector3df(-140, 50, 0));
    mLightSN[0]->getLightData().DiffuseColor.set(0.9f, 1, 0.9f);
    mLightSN[1] = mSmgr->addLightSceneNode(0, core::vector3df(40, 3, 0));
    mLightSN[1]->getLightData().DiffuseColor.set(0.7f, 0.9f, 1.0f);
    //mSmgr->addCubeSceneNode(1);

    SMaterial planeMaterial;
    planeMaterial.Shininess = 0;
    planeMaterial.SpecularColor.set(0);
    planeMaterial.DiffuseColor.set(0xff708090);
    planeMaterial.AmbientColor.set(0xff708090);
    IMesh *planeMesh = mSmgr->addHillPlaneMesh("wat", dimension2df(4, 11), dimension2d<u32>(100, 15), &planeMaterial, 0.25f, core::dimension2df(20, 3))->getMesh(0);
    //ISceneNode *planeSN = mSmgr->addMeshSceneNode(planeMesh);
    ISceneNode *planeSN = mSmgr->addWaterSurfaceSceneNode(planeMesh, 0.5f, 300.f, 10.f);
    planeSN->setPosition(vector3df(100, 0.25f, 0));
    planeSN->setMaterialType(EMT_TRANSPARENT_ADD_COLOR);
    planeSN->getMaterial(0).setTexture(0, skyDomeTex);
    planeSN->getMaterial(0).TextureLayer[0].TextureWrap = video::ETC_REPEAT;
    planeSN->setAutomaticCulling(EAC_OFF);
    //planeSN->getMaterial(0).setFlag(video::EMF_FOG_ENABLE, true);

    mWallSN = mSmgr->addEmptySceneNode();
    IMesh *wallMesh = mSmgr->addHillPlaneMesh("wall", dimension2df(6, 5), dimension2du(30, 4), &planeMaterial, 0, core::dimension2df(1, 1));
    ISceneNode *wallSN = mSmgr->addWaterSurfaceSceneNode(wallMesh, 1.f, 210.f, 7.3f, mWallSN);
    mWallWSN = wallSN;
    wallSN->setRotation(vector3df(0, -90, -90));
    //mNoteXOffset = -100;
    float wallStartPos = -100;
    wallSN->setPosition(vector3df(wallStartPos, 6, 0));
    wallSN->setMaterialType(video::EMT_SOLID);
    wallSN->getMaterial(0).setTexture(0, skyDomeTex);
    wallSN->getMaterial(0).TextureLayer[0].TextureWrap = video::ETC_MIRROR;
    wallSN->setMaterialFlag(video::EMF_LIGHTING, false);
    wallSN->getMaterial(0).getTextureMatrix(0).setScale(vector3df(1, 1, 1));
//    wallSN->getMaterial(0).setFlag(video::EMF_FOG_ENABLE, true);

    mSmgr->loadScene("data/test_prt.irr");
    ISceneNode *wallprt = mSmgr->getSceneNodeFromName("wallprt");
    wallprt->setParent(mWallSN);
    wallprt->setPosition(vector3df(wallStartPos, -1, 0));
    IParticleSystemSceneNode *prt = static_cast<IParticleSystemSceneNode *>(wallprt);
    prt->addAffector(prt->createGravityAffector(vector3df(0, -0.0011f, 0), 8000));
    prt->addAffector(prt->createFadeOutParticleAffector(video::SColor(0), 8000));
    prt->addAffector(prt->createScaleParticleAffector(core::dimension2df(15, 15)));

    video::ITexture *gndTex = mVideo->getTexture("data/ground.png");
    SMaterial gndMaterial;
    gndMaterial.AmbientColor.set(0xffffffff);
    //IMesh *groundMesh = mSmgr->addHillPlaneMesh("gnd", dimension2df(600, 200), dimension2d<u32>(1, 1), &gndMaterial, 0, dimension2df(0, 0), dimension2df(9, 2));
    //ISceneNode *groundSN = mSmgr->addMeshSceneNode(groundMesh);
    ISceneNode *groundSN = mSmgr->addTerrainSceneNode("data/ground-height.png", 0, -1, vector3df(-100, -20, -180), vector3df(0, 0, 0), vector3df(9, 0.05f, 3.f));
    //groundSN->setPosition(vector3df(150, -10, 0));
    groundSN->setMaterialTexture(0, gndTex);
    groundSN->setMaterialFlag(video::EMF_LIGHTING, false);
    groundSN->getMaterial(0).getTextureMatrix(0).setScale(vector3df(9, 3, 1));
    groundSN->setAutomaticCulling(EAC_OFF);

    IMesh *hexTmp = mSmgr->getMesh("data/hex.obj");
    mHexMesh = mSmgr->getMeshManipulator()->createMeshUniquePrimitives(hexTmp);


    /*
    array<ISceneNode *> nodes;
    u32 a;

    mSmgr->getSceneNodesFromType(ESNT_LIGHT, nodes);
    for (a = 0; a < nodes.size(); ++a)
    {
        ILightSceneNode *lsn = (ILightSceneNode *)nodes[a];
        SLight &ld = lsn->getLightData();
        // ...
    }

    nodes.clear();
    mSmgr->getSceneNodesFromType(ESNT_CUBE, nodes);
    for (a = 0; a < nodes.size(); ++a)
    {
        ISceneNode *sn = nodes[a];
        if (sStringStartsWith(sn->getName(), "prefix_"))
        {
            s32 id = sn->getID();
            //sn->getAbsolutePosition() ...
            // ...
        }
    }
    */

    mCamPos = mCamera->getPosition();
    mCamTargetPos = mCamera->getTarget();
    mWallPos = mWallSN->getPosition();
    mLightPos[0] = mLightSN[0]->getPosition();
    mLightPos[1] = mLightSN[1]->getPosition();

    IAnimatedMesh *playerMesh = mSmgr->getMesh("data/surfer.3ds");
    mPlayerMeshSceneNode = mSmgr->addAnimatedMeshSceneNode(playerMesh);
    mPlayerMeshSceneNode->setScale(vector3df(5, 5, 5));
    for (u32 a = 0; a < mPlayerMeshSceneNode->getMaterialCount(); ++a)
    {
        video::SColor diffuse = mPlayerMeshSceneNode->getMaterial(a).DiffuseColor;
        mPlayerMeshSceneNode->getMaterial(a).EmissiveColor.set(0xff, diffuse.getRed() / 2, diffuse.getGreen() / 2, diffuse.getBlue() / 2);
    }
    //for (u32 a = 0; a < mPlayerMeshSceneNode->getMaterialCount(); ++a)
    //    mPlayerMeshSceneNode->getMaterial(a).setFlag(video::EMF_LIGHTING, false);
    //mPlayerMeshSceneNode->addShadowVolumeSceneNode(0, -1, false);
    mPlayerPos = mPlayerMeshSceneNode->getPosition();

    mSmgr->setShadowColor(SColor(0x14001040));

    //mVideo->setFog(SColor(0, 140, 190, 230), true, 50, 150, 0.01f);
    
    mLastMousePos.set(WIDTH / 2, HEIGHT / 2);

    mScore = 0;

    //loadSongNotes(mSongNotes, "data/music1.txt");

    // spawn note cubes
    mSongNoteOffset = -72;
    //int noteOffset = -72;
    float noteSpacingDepthScale = 1.5f;
    float noteSpacingHorizScale = 1.1f;
    /*
    for (UI32 a = 0; a < mSongNotes.size(); ++a)
    {
        if (mSongNotes[a] < 0)
            continue;
        float sceneXOffset = -25; //mNoteXOffset;
        vector3df pos(sceneXOffset + a * noteSpacingDepthScale, 0.1f, (mSongNotes[a] + noteOffset) * noteSpacingHorizScale);
        //ISceneNode *sn = mSmgr->addCubeSceneNode(1, 0, a + 1000, pos);
        ISceneNode *sn = mSmgr->addMeshSceneNode(mHexMesh, 0, a + 1000, pos);
        //sn->getMaterial(0).setFlag(video::EMF_FOG_ENABLE, true);
        sn->getMaterial(0).setFlag(video::EMF_GOURAUD_SHADING, false);
    }
    */
    mNoteSpacingDepthScale = noteSpacingDepthScale;
    mNoteSpacingHorizScale = noteSpacingHorizScale;

    return mFont != NULL && mButtonStyle != NULL;
}


void MainState::deinit()
{
    UI32 a;
    delete mFont;
    for (a = 0; a < mImages.size(); ++a)
        delete mImages[a];
    mImages.clear();

    FMUSIC_StopAllSongs();
    for (a = 0; a < LEVEL_COUNT; ++a)
        FMUSIC_FreeSong(sMusics[a]);
    FMUSIC_FreeSong(sTitleMusic);
    for (a = 0; a < AUDIO_CHANNELS; ++a)
        FSOUND_StopSound(a);
    for (a = 0; a < SAMPLE_COUNT; ++a)
        FSOUND_Sample_Free(sSamples[a]);
    FSOUND_Close();
}


void MainState::loadSongNotes(MainState::SongNotes &songNotes, const char *filename)
{
    songNotes.clear();

    FILE *fp;
    fp = fopen(filename, "r");
    char buf[1024];
    const char COLUMN_ID = '|';
    const int OCTAVE_NOTE_COUNT = 12;
    static char *notes[OCTAVE_NOTE_COUNT] = { "C-", "C#", "D-", "D#", "E-", "F-", "F#", "G-", "G#", "A-", "A#", "B-" };
    while (fgets(buf, 1024, fp))
    {
        int column = 0;
        if (buf[column] != COLUMN_ID)
            continue;
        bool found = false;
        for (int a = 0; a < OCTAVE_NOTE_COUNT; ++a)
        {
            if (memcmp(notes[a], &buf[column + 1], 2) == 0)
            {
                int octave = buf[column + 3] - '0';
                int note = octave * OCTAVE_NOTE_COUNT + a;
                songNotes.push_back(note);
                found = true;
                break;
            }
        }
        if (!found)
            songNotes.push_back(-1);
    }
    fclose(fp);
}



void MainState::onActivate(UI32 time)
{
    AppState::onActivate(time);
    mStartTime = 0; // first call to update will do actual initialization
    mPaused = false;
}


bool MainState::onKeyUp(const irr::SEvent::SKeyInput & event)
{
    if (event.Key == KEY_LEFT)
        mKeyLeft = false;
    if (event.Key == KEY_RIGHT)
        mKeyRight = false;
    return false;
}

bool MainState::onKeyDown(const irr::SEvent::SKeyInput &event)
{
    if (event.Key == KEY_ESCAPE)
        App::getInstance()->setAlive(false);

    if (event.Key == KEY_LEFT)
        mKeyLeft = true;
    if (event.Key == KEY_RIGHT)
        mKeyRight = true;

    return false;
}

bool MainState::onMouseDown(const irr::SEvent::SMouseInput & /*event*/)
{
    if (mTitleUIActive || mEndingUIActive)
        mIMGUI.onMouseDown();

    return false;
}

bool MainState::onMouseUp(const irr::SEvent::SMouseInput & /*event*/)
{
    mIMGUI.onMouseUp();
    return false;
}

bool MainState::onMouseMotion(const irr::SEvent::SMouseInput &event)
{
    mLastMousePos.set(event.X, event.Y);
    mIMGUI.onMouseMotion(event.X, event.Y);
    return false;
}

bool MainState::onMouseWheel(const irr::SEvent::SMouseInput & /*event*/)
{
    return false;
}


bool MainState::update(UI32 time)
{
    bool result = AppState::update(time);
    if (!result)
        return false; // game is paused...

    if (mTitleUIActive)
    {
        return false;
    }

    if (mInGame && mSongPosition >= (int)mSongNotes.size())
    {
        mInGame = false;
        mEndingUIActive = true;
        FSOUND_PlaySound(FSOUND_FREE, sSamples[SAMPLE_END]);
        FMUSIC_StopSong(sMusic);
        FMUSIC_PlaySong(sTitleMusic);
    }

    if (!mInGame)
        return false;

    // Following is executed only when game is not paused & game is running.

    /*
    if (mPendingCamMovement > 0)
    {
        int camMovement = mPendingCamMovement;
        mPendingCamMovement = 0;
        float movement = camMovement * mNoteSpacingDepthScale;
        mCamPos.X += movement;
        mCamTargetPos.X += movement;
        mWallPos.X += movement;
        mLightPos[0].X += movement;
        mLightPos[1].X += movement;
    }
    */

    mCamPos.X += GAME_SPEED;
    mCamTargetPos.X += GAME_SPEED;
    mWallPos.X += GAME_SPEED;
    mLightPos[0].X += GAME_SPEED;
    mLightPos[1].X += GAME_SPEED;

    mCamera->setPosition(blurPos(mCamera->getPosition(), mCamPos));
    mCamera->setTarget(blurPos(mCamera->getTarget(), mCamTargetPos));
    mWallSN->setPosition(blurPos(mWallSN->getPosition(), mWallPos));
    mLightSN[0]->setPosition(blurPos(mLightSN[0]->getPosition(), mLightPos[0]));
    mLightSN[1]->setPosition(blurPos(mLightSN[1]->getPosition(), mLightPos[1]));

    mPlayerPos.X += GAME_SPEED / 2;
    mPlayerMeshSceneNode->setPosition(blurPos(mPlayerMeshSceneNode->getPosition(), mPlayerPos, 0.04f));
    const int maxCDist = 15;
    if (mKeyLeft && mPlayerPos.Z > -maxCDist)
        mPlayerPos.Z -= GAME_SPEED;
    if (mKeyRight && mPlayerPos.Z < maxCDist)
        mPlayerPos.Z += GAME_SPEED;

    mSongRow = FMUSIC_GetRow(sMusic);
    while (mSongPrevRow != mSongRow)  // heh weirdo
    {
        mSongPrevRow = mSongRow;
        //--mPendingRowUpdates;
        ++mSongPosition;
        if (mSongPosition < 0 || mSongPosition >= (int)mSongNotes.size())
            continue;   // end of tune or something
        if (mSongNotes[mSongPosition] < 0)
            continue;   // song row doesn't have a note
        float spawnXOffset = -8;
        vector3df pos(mCamPos.X + spawnXOffset, 0.1f, (mSongNotes[mSongPosition] + mSongNoteOffset) * mNoteSpacingHorizScale);
        ISceneNode *sn = mSmgr->addMeshSceneNode(mHexMesh, 0, mSongPosition + 1000, pos);
        //sn->getMaterial(0).setFlag(video::EMF_FOG_ENABLE, true);
        sn->getMaterial(0).setFlag(video::EMF_GOURAUD_SHADING, false);
        mActiveNoteNodes.push_back(sn);
    }

    aabbox3df playerBB = mPlayerMeshSceneNode->getTransformedBoundingBox();
    aabbox3df wallBB = mWallWSN->getTransformedBoundingBox();
    if (playerBB.intersectsWithBox(wallBB))
    {
        mInGame = false;
        mEndingUIActive = true;
        mDied = true;
        FSOUND_PlaySound(FSOUND_FREE, sSamples[SAMPLE_DEATH]);
        FMUSIC_StopSong(sMusic);
        FMUSIC_PlaySong(sTitleMusic);
    }
    for (u32 a = 0; a < mActiveNoteNodes.size(); )
    {
        ISceneNode *noteNode = mActiveNoteNodes[a];
        aabbox3df noteBB = noteNode->getTransformedBoundingBox();
        if (noteBB.intersectsWithBox(playerBB))
        {
            ++mScore;
            mPlayerPos.X += 3;
            int chn = FSOUND_PlaySound(FSOUND_FREE, sSamples[SAMPLE_PICKUP]);
            FSOUND_SetVolume(chn, 0x80);
            noteNode->remove();
            mActiveNoteNodes.erase(mActiveNoteNodes.begin() + a);
        }
        else if (noteBB.intersectsWithBox(wallBB))
        {
            noteNode->remove();
            mActiveNoteNodes.erase(mActiveNoteNodes.begin() + a);
        }
        else
            ++a;
    }

    return true;
}


vector3df MainState::blurPos(const core::vector3df &orgPos, const core::vector3df &targetPos, float factor)
{
    return orgPos * (1 - factor) + targetPos * factor;
}


void MainState::render(irr::IrrlichtDevice *device)
{
    assert(device);
    irr::video::IVideoDriver *driver = device->getVideoDriver();
    assert(driver);
    irr::scene::ISceneManager *smgr = device->getSceneManager();

    driver->beginScene(true, true, video::SColor(0x00001122));

    smgr->drawAll();

    Graphics *g = App::getInstance()->getGraphics();
    g->beginRender();
    mIMGUI.begin(mTime);

    bool startNewLevel = false;
    int newLevelNum = 0;
    if (mTitleUIActive)
    {
        mFont->drawString(WIDTH / 2, 150, "tsunami warrior", IF_ALIGN_HORIZ_CENTER, Color(0xffffffff));
        mFont->drawString(WIDTH / 2, 180, "surfer", IF_ALIGN_HORIZ_CENTER, 0.7f, Color(0x80ffffff));

        mFont->drawString(WIDTH / 2, 250, "collect the notes using left & right arrow keys", IF_ALIGN_HORIZ_CENTER, 0.7f, Color(0xaaffffff));
        mFont->drawString(WIDTH / 2, 280, "you must escape looming doom of the wave", IF_ALIGN_HORIZ_CENTER, 0.7f, Color(0xaaffffff));

        mButtonStyle->setTextScale(0.7f);
        if (mIMGUI.button(WIDTH / 2 - mButtonStyle->getWidth() - 10, 350, IMGUI_AI_COUNTER, mButtonStyle, 0, 0, "Play Level A"))
        {
            startNewLevel = true;
            newLevelNum = 0;
        }
        if (mIMGUI.button(WIDTH / 2 + 10, 350, IMGUI_AI_COUNTER, mButtonStyle, 0, 0, "Play Level B"))
        {
            startNewLevel = true;
            newLevelNum = 1;
        }
        mButtonStyle->setTextScale(1.0f);

        if (startNewLevel)
        {
            mTitleUIActive = false;
            mInGame = true;
            mStartTime = 0;
            mPendingRowUpdates = 0;
            mSongPrevRow = -1;
            mSongRow = 0;
            mSongPosition = -1;
            for (u32 a = 0; a < mActiveNoteNodes.size(); ++a)
                mActiveNoteNodes[a]->remove();
            mActiveNoteNodes.clear();
            mDied = false;
            mPlayerMeshSceneNode->setPosition(vector3df(-6, 0, 0));
            mPlayerPos = mPlayerMeshSceneNode->getPosition();
            sMusic = sMusics[newLevelNum];
            char buf[100];
            sprintf(buf, "data/music%d.txt", newLevelNum);
            loadSongNotes(mSongNotes, buf);
            //FSOUND_PlaySound(FSOUND_FREE, sSamples[SAMPLE_START]);
            FMUSIC_StopSong(sTitleMusic);
            FMUSIC_PlaySong(sMusic);
        }
        if (mIMGUI.button(WIDTH / 2 - mButtonStyle->getWidth() / 2, 450, IMGUI_AI_COUNTER, mButtonStyle, 0, 0, "Quit"))
            App::getInstance()->setAlive(false);

        mFont->drawString(WIDTH / 2, HEIGHT - 5, "Copyright 2009 Jetro Lauha - www.jet.ro", IF_ALIGN_BOTTOM_HCENTER, 0.5f, Color(0x80ffffff));
    }
    else if (mInGame)
    {
        char buf[100];
        sprintf(buf, "%d", mScore);
        mFont->drawString(40, 40, "score", IF_ALIGN_TOP_LEFT, 0.7f, Color(0x88ffffff));
        mFont->drawString(40, 60, buf, IF_ALIGN_TOP_LEFT, Color(0xff000000));

        #ifdef _DEBUG
        sprintf(buf, "songPos: %d", mSongPosition);
        mFont->drawString(0, 500, buf, IF_ALIGN_TOP_LEFT, 0.7f, Color(0xffff8080));
        #endif
    }
    else if (mEndingUIActive)
    {
        char *title = "You made it!";
        char buf[100];
        sprintf(buf, "%d", mScore * (mDied ? 1 : 1000000));
        if (mDied)
            title = "Failboat, you tumbled over!";
        mFont->drawString(WIDTH / 2, 200, title, IF_ALIGN_HORIZ_CENTER, Color(0xff000000));
        mFont->drawString(WIDTH / 2, 300, "Final score with bonus multiplier:", IF_ALIGN_HORIZ_CENTER, 0.6f, Color(0x80000000));
        mFont->drawString(WIDTH / 2, 350, buf, IF_ALIGN_HORIZ_CENTER, 0.8f, Color(0xcc000000));
        if (mIMGUI.button(WIDTH / 2 - mButtonStyle->getWidth() / 2, 450, IMGUI_AI_COUNTER, mButtonStyle, 0, 0, "Quit"))
        {
            // uh.. ran out of time - these aren't enough to reset the state properly,
            // should refactor out the level reset stuff from MainState::init()
            //mDied = false;
            //mTitleUIActive = true;
            //mEndingUIActive = false;
            App::getInstance()->setAlive(false);
        }
    }

    mIMGUI.end();
    g->endRender();

    driver->endScene();
}
