/* Copyright 2009 Jetro Lauha - http://iki.fi/jetro/
 *
 * $Id: MainState.h 5 2009-04-19 23:02:11Z tonic $
 * $Revision: 5 $
 */

#ifndef MAINSTATE_H_INCL
#define MAINSTATE_H_INCL


#include <vector>

//using namespace std;
using namespace irr;
using namespace scene;
using namespace core;
using namespace video;


class MainState : public AppState
{
public:

    MainState();
    virtual ~MainState();

    // From AppState:

    virtual bool init();
    virtual void deinit();

    virtual bool update(UI32 time);
    virtual void render(irr::IrrlichtDevice *device);

    virtual void onActivate(UI32 time);
    virtual void onDeactivate() {}

    virtual bool onKeyDown(const irr::SEvent::SKeyInput &event);
    virtual bool onKeyUp(const irr::SEvent::SKeyInput &event);
    virtual bool onMouseDown(const irr::SEvent::SMouseInput &event);
    virtual bool onMouseUp(const irr::SEvent::SMouseInput &event);
    virtual bool onMouseMotion(const irr::SEvent::SMouseInput &event);
    virtual bool onMouseWheel(const irr::SEvent::SMouseInput &event);
    // Irr is lacking focus lost/gain events..
    //virtual void onFocusLost(const SDL_ActiveEvent &event) {}
    //virtual void onFocusGain(const SDL_ActiveEvent &event) {}

    // Local
    void newMusicOrder(int order);
    void newMusicRow(int row);

protected:

    enum IMAGE_ID
    {
        IMG_BUTTON = 0,
        IMG_BUTTON_HOT,
        IMG_BUTTON_ACTIVE,
        IMG_BUTTON_HOT_ACTIVE
    };

    typedef std::vector<int> SongNotes;

    void loadSongNotes(SongNotes &songNotes, const char *filename);
    core::vector3df blurPos(const core::vector3df &orgPos, const core::vector3df &targetPos, float factor = 0.1f);

    App *mApp;

    IMGUI mIMGUI;
    IMGUIImageButtonStyle *mButtonStyle;

    Font *mFont;
    std::vector<Image *> mImages;
    C8 *mLabelText;

    IrrlichtDevice *mDevice;
    IVideoDriver *mVideo;
    ISceneManager *mSmgr;

    ICameraSceneNode *mCamera;

    vector2di mLastMousePos;

    bool mTitleUIActive;
    bool mInGame;
    bool mEndingUIActive;

    IAnimatedMeshSceneNode *mPlayerMeshSceneNode;

    s32 mScore;
    bool mDied;
    bool mKeyLeft, mKeyRight;

    SongNotes mSongNotes;
    int mSongNoteOffset;
    volatile int mSongPosition, mSongPrevRow, mSongRow;
    volatile int mPendingRowUpdates;

    std::vector<ISceneNode *> mActiveNoteNodes;

    float mNoteSpacingDepthScale, mNoteSpacingHorizScale;//, mNoteXOffset;
    scene::ISceneNode *mWallSN, *mWallWSN;
    scene::ILightSceneNode *mLightSN[2];

    core::vector3df mCamPos, mCamTargetPos, mWallPos, mLightPos[2], mPlayerPos;

    scene::IMesh *mHexMesh;
};


#endif // !MAINSTATE_H_INCL
